MODULE EmxMouse;

(* This program is for TopSpeed Modula-2 V3.10 *)
(* Written 1994 by Jrn Sierwald               *)
(* Sierwald@tu-harburg.d400.de                 *)

IMPORT IO,SYSTEM,Lib,MsMouse,TSR;

(*# data(stack_size=>500H) *)

VAR p [0000H:011CH] : ADDRESS; (* Int 47H *)
VAR Ticks [0040H:006CH] : LONGCARD;
VAR Screen [0B800H:0] : CARDINAL;

CONST QueueSize = 32;

TYPE
  MouseEvent = RECORD
    buttons     : CARDINAL;
    x,y         : CARDINAL;
  END;

TYPE
  Event = RECORD
    semaphore : CARDINAL;
    time      : LONGCARD;
    mouse     : MouseEvent;
  END;

TYPE
  SharedRec = RECORD
    magic               : LONGCARD;
    curMouseSemaphore   : CARDINAL;
    curMouse            : MouseEvent;
    eventSemaphore      : CARDINAL;
    eventQueue          : ARRAY [0..QueueSize-1] OF Event;
    eventQHead          : CARDINAL;
    eventQTail          : CARDINAL;
    buttonCount         : CARDINAL;
  END;

(*
  SharedRec is shared by this program and the emx/gcc/dos program.

  The ringbuffer handling has been changed. emx programs cannot
  disable the interupts. Thus, the buffer must be protected by a number
  of flags/semaphores. The eventCount variable has been removed.

  magic :
    contains 3141592654.
    the 32Bit program detects the emxmouse.exe by this value.
  curMouseSemaphore :
    0 : This program may write the curMouse member.
    1 : 32Bit program is reading the curMouse member,
        this program must not write the curMouse member.
  curMouse :
    the current mouse position. written by this program and
    read by the 32Bit program
  eventSemaphore :
    0 : This program may write the eventQueue.
    1 : 32Bit program not yet loaded, this program may not write the
        eventQueue.
  eventQueue :
    The ringbuffer.
    Every array element has its own flag/semaphore:
      0 : empty
      1 : full
  eventQHead :
    The 32Bit program will read from this position.
  eventQTail :
    This program will write to this position.
  buttonCount :
    # of buttons.

*)

VAR v : SharedRec;

TYPE
      IPROC       = PROCEDURE();
VAR
      Irpt[0:0] : ARRAY [0..255] OF IPROC ;

PROCEDURE Activate();
VAR R : SYSTEM.Registers;
BEGIN

  R.AX:=021H;
  Lib.Intr(R,033H);

  IO.WrLn;
  IO.WrStr("Mouse TSR for emx is being unloaded");
(*  IO.WrChar(7C); *)
  IO.WrLn;
  TSR.DeInstall;
END Activate;

(*# save *)
(*# call(o_a_size=>on,o_a_copy=>on,c_conv=>on,reg_param=>()) *)
PROCEDURE Func ( a : ARRAY OF CHAR );
BEGIN
  IO.WrStr(a); IO.WrChar(7C); IO.WrLn;
END Func;

(*# restore *)

(*
/*
#pragma saveregs
void huge TEventQueue::mouseInt()
{
    unsigned flag = _AX;
    MouseEventType tempMouse;

    tempMouse.buttons = _BL;
    tempMouse.doubleClick = False;
    tempMouse.where.x = _CX >> 3;
    tempMouse.where.y = _DX >> 3;

    if( (flag & 0x1e) != 0 && eventCount < eventQSize )
        {
        eventQTail->what = *Ticks;
        eventQTail->mouse = curMouse;
        if( ++eventQTail >= eventQueue + eventQSize )
            eventQTail = eventQueue;
        eventCount++;
        }

    curMouse = tempMouse;
    mouseIntFlag = True;
}
*/
  *)
(*----------------------------------------------------------------------*)
(* Actual Mouse Interrupt Handler

   This is not really an interupt handler.
   It is called by the mouse driver and return with RTS

*)

(* pragmas for interrupt handler *)
(*# save,
         call ( same_ds   => off )

*)

PROCEDURE MouseInt ( AX,BX,CX,DX : CARDINAL );
VAR
  flag : CARDINAL;
  tempMouse : MouseEvent;
BEGIN

  flag:=AX;
  tempMouse.buttons:=CARDINAL(BITSET(BX) * BITSET( 0FH ) );
  tempMouse.x:= CX >> 3;
  tempMouse.y:= DX >> 3;
  IF v.eventSemaphore=0 THEN
    IF ( CARDINAL(BITSET(flag) * BITSET(01EH)) # 0 ) THEN
      IF v.eventQueue[(v.eventQTail+1) MOD QueueSize].semaphore = 0 THEN
        v.eventQueue[v.eventQTail].time := Ticks;
        v.eventQueue[v.eventQTail].mouse := tempMouse;
        v.eventQueue[v.eventQTail].semaphore := 1;
        v.eventQTail:=(v.eventQTail+1) MOD QueueSize;
      END;
    END;
  END;
  IF v.curMouseSemaphore=0 THEN
    v.curMouse:=tempMouse;
  END;
  Screen:=Screen+12345;
END MouseInt;
(*# restore *)

VAR R : SYSTEM.Registers;
    is : MsMouse.MsInterrupt;
    i : CARDINAL;
BEGIN
(* Set up the ring buffer *)

  v.eventQHead:=0;
  v.eventQTail:=0;
  v.magic:=3141592654;
  v.curMouseSemaphore:=0;
  v.eventSemaphore:=1;

  v.curMouse.buttons:=0;
  v.curMouse.x:=0;
  v.curMouse.y:=0;

  FOR i:=0 TO QueueSize-1 DO
    v.eventQueue[i].semaphore:=0;
  END;

(* Call Mouse handler *)

  IF Irpt[033H]=IPROC([0:0]) THEN
    IO.WrStr("Mousedriver not found");
    IO.WrLn;
    RETURN;
  END;

  R.AX:=0;
  Lib.Intr(R,033H);
  v.buttonCount:=R.BX;
  IF v.buttonCount=07FFFH THEN RETURN; END;
  IF v.buttonCount=0FFFFH THEN v.buttonCount:=2; END;
  R.AX:=4; (* Set Mouse Pos *)
  R.CX:=0;
  R.DX:=0;
  Lib.Intr(R,033H);

  R.AX:=7;
  R.CX:=0;
  R.DX:=80 << 3;
  Lib.Intr(R,033H);

  R.AX:=8;
  R.CX:=0;
  R.DX:=25 << 3;
  Lib.Intr(R,033H);

  is.IntFunc:=MouseInt;
  is.mask:=0FFFFH;
  MsMouse.SetInterrupt(is);
(*
  R.AX:=1;  (* show mouse *)
  Lib.Intr(R,033H);
*)

  p:=ADR(v);
  IO.WrLn;
  IO.WrStr("Mouse TSR for emx is installed");
(*  IO.WrChar(7C); *)
  IO.WrLn;
  IO.WrStr("(p) 1994 Jrn Sierwald, public domain");
  IO.WrLn;
  IO.WrStr("Unload with Ctrl-LeftShift-A");
  IO.WrLn;
  IO.WrCard(Seg(v),1);
  IO.WrChar(":");
  IO.WrCard(Ofs(v),1);
  IO.WrLn;

(* Install TSR *)

  TSR.Install(Activate,TSR.KBFlagSet{TSR.LShift,TSR.Ctrl},30,32);



END EmxMouse.
